#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#define pi 3.141593



/* The function Ku is the kernel used. See (2.2) in the paper. */


double Ku(double a,double h, double sigma)
{
	double n0=100.0;
	int j;
	int i;
	double temp;
	double temp9;
	
	for (i=1;i<=100;i++){
		temp9 +=(cos((i-0.5)*a/n0)*pow((1-pow((i-0.5)/n0,2)),3)*exp(0.5*(((i-0.5)/n0)*((i-0.5)/n0))*(sigma*sigma)/(h*h)));
	}
	temp=(1/(n0*pi))*temp9;
	temp9=0;
	return temp;
}





/* The function Ke2 is the second derivative of the kernel used. */

double Ke2(double ael, double h, double sig){
	
	double n0=100.0;
	int j;
	int i;
	double temp;
	double temp9;
	
	for (i=1;i<=100;i++){
		temp9 += -((i-0.5)/n0)*((i-0.5)/n0)*(cos((i-0.5)*ael/n0)*pow((1-pow((i-0.5)/n0,2)),3)*exp(0.5*(((i-0.5)/n0)*((i-0.5)/n0))*(sig*sig)/(h*h)));
	}
	temp=(1/(n0*pi))*temp9;
	temp9=0;
	return temp;
}




/* ebbsi() is the main function that is called from R. It uses all the above functions 
 and it takes the following arguments: "w" is the contaminated sample for which 
 we want to estimate the density, "n" is the sample size, "G" is the equally-spaced grid
 of points at which we want to estimate the desnity, "ng" is the size of the grid G, "H1" is 
 the grid of bandwidths, "M1" is the size of the grid H1, "sigma" is the parameter of the
 Laplace(sigma) error distribution, "hglob" is the h_GPI bandwidth, "J_11", "J_12", "J_21" and "J_22" 
 correspond to a_1, a_2, beta_1 and beta_2 of page 12, section 4.1 in the papaer respectively. 
 Finally, ans_h is a vector where the result is going to be stored. After the function ipii()
 is compiled, the local bandwidth is saved in "ans_h".  
 */


void ebbsi(double *w, int *n, double *G, int *ng, double *H1, int *M1,double *sigma,double *hglob,int *J_11, int *J_12, int *J_21, int *J_22, double *ans_h){

	int i,j,k,l,g;
	double f_tild;
	double h_0,I0,I2,I4;
	double int_med;
	double sumKe2;
	double temp_val;
	int n_0=100;
	double a,b,temp0;
	double bias_est,varest,int_hm;
	double mse[*ng][*M1];
	
	
	
	for (g=0;g<=(*ng-1);g++){
		for (i=0;i<=(*M1-1);i++){
			
			/* Calculate the bias coefficient using the EBBS method - integral form. We use numerical integration. See section 4.1 in the paper for implementation details */	

			if(H1[i]<=*hglob*0.6666667){
				
				a=H1[i]*pow(1.1,- *J_11);
				b=H1[i]*pow(1.1,*J_12);
				
				int_hm=0;
				int_med=0;
				for (l=1;l<=n_0;l++){
					h_0 = a + (l-0.5)*(b-a)/n_0;
					sumKe2=0;
					
					for(k=0;k<=(*n-1);k++){
						temp_val = (G[g]-w[k])/h_0;
						sumKe2 = sumKe2 + Ku(temp_val,h_0,*sigma);
					}
					
					int_med = int_med + ((b-a)/n_0)*(1/(*n * h_0))*sumKe2;
					int_hm = int_hm + ((b-a)/n_0)*(h_0 / *n)*sumKe2;
				}

				I0 = b-a;
				I2 = (b*b*b - a*a*a)/3;
				I4 = (b*b*b*b*b - a*a*a*a*a)/5;
				
				
				f_tild = (I0*int_hm - I2*int_med) / (I4*I0 - I2*I2); 

			}	
				
			else{
				
				a=H1[i]*pow(1.1,- *J_21);
				b=H1[i]*pow(1.1,*J_22);
				
				int_hm=0;
				int_med=0;
				for (l=1;l<=n_0;l++){
					h_0 = a + (l-0.5)*(b-a)/n_0;
					sumKe2=0;

					for(k=0;k<=(*n-1);k++){
						temp_val = (G[g]-w[k])/h_0;
						sumKe2 = sumKe2 + Ku(temp_val,h_0,*sigma);
					}
					
					int_med = int_med + ((b-a)/n_0)*(1/(*n * h_0))*sumKe2;
					int_hm = int_hm + ((b-a)/n_0)*(h_0 / *n)*sumKe2;
				}
				
				
				I0 = b-a;
				I2 = (b*b*b - a*a*a)/3;
				I4 = (b*b*b*b*b - a*a*a*a*a)/5; 
				
				
				f_tild = (I0*int_hm - I2*int_med) / (I4*I0 - I2*I2); 
			
			
			}
				
		
			
			
			/* Calculate the MSE using the estimated bias coefficient */ 			
			
			
			bias_est = H1[i]*H1[i]*f_tild;
			
			varest=0;

			for (j=0;j<=(*n-1);j++){
				temp0 = (G[g]-w[j])/H1[i];
				varest += (1/(*n * *n * H1[i] * H1[i]))*pow(Ku(temp0,H1[i],*sigma),2);
			}
			
			mse[g][i] = bias_est*bias_est + varest;
			
			
			
		}
	}

	
	
	
	/* Minimize the MSE */ 			
	
	
	int M1_1;
	M1_1 = *M1-1;
	double hopt[*ng];
	
	
	for (g=0;g<=(*ng-1);g++){
		for (i=1;i<=(*M1-2);i++){
			
			if((mse[g][i] < mse[g][i-1]) && (mse[g][i] < mse[g][i+1])){ 
				hopt[g]=H1[i];
				break;
			}
			else{
				if(mse[g][0] < mse[g][M1_1]) hopt[g]=H1[0];
				else hopt[g]=H1[M1_1];
			}
			
			
			
		}
	}
	
	
	
	
	/* Save the resulting local bandwidth in ans_h */ 			
	
	for (g=0;g<=(*ng-1);g++){
		ans_h[g] = hopt[g];
	}
	
}
	
	
	
	
	
	
	
	
	
	
	
	
	




